from enum import Enum


class LegacyErrorCodeEnum(str, Enum):
    pass


class ErrorCodeEnum(str, Enum):
    """
    Enum of error codes.
    """

    # General errors

    MISSING_AUTHORIZATION_HEADER = 40001
    MISSING_TOKEN = 40002
    INVALID_TOKEN = 40003
    TOKEN_HAS_EXPIRED = 40004
    MALFORMED_TOKEN_PAYLOAD = 40005
    INVALID_CLIENT_ID = 40006
    INSUFFICIENT_SCOPE = 40007
    INTERNAL_SERVER_ERROR = 41000
    RCS_REQUEST_TIMEOUT = 41001
    REVOCATION_FAILED = 41002
    THROTTLED_BY_MESSAGE_RATE = 41003
    RCS_SERVER_BUSY = 41004
    RCS_SERVER_TEMPORARILY_UNAVAILABLE = 41005
    SESSION_DOES_NOT_EXIST = 41006
    EXPIRED_BEFORE_SESSION_ESTABLISHMENT = 41007
    SESSION_ALREADY_EXPIRED = 41008
    IMDN_RECEIVED_EVEN_ALREADY_REVOKED = 41010
    SUBSCRIBE_TIMEOUT = 41100
    USER_NOT_FOUND = 41101
    USER_NOT_FOUND_ = 41200
    MESSAGE_NOT_ACCEPTABLE = 41201
    USER_IS_NOT_CAPABLE_FOR_TEXT = 41210
    USER_IS_NOT_CAPABLE_FOR_FT = 41211
    USER_IS_NOT_CAPABLE_FOR_RICHCARD = 41212
    USER_IS_NOT_CAPABLE_FOR_XBOTMESSAGE_1_0 = 41220
    USER_IS_NOT_CAPABLE_FOR_XBOTMESSAGE_1_1 = 41221
    USER_IS_NOT_CAPABLE_FOR_XBOTMESSAGE_1_2 = 41222
    USER_IS_NOT_CAPABLE_FOR_OPENRICHARD_1_0 = 41230
    USER_IS_NOT_CAPABLE_FOR_OPENRICHARD_1_1 = 41231
    USER_IS_NOT_CAPABLE_FOR_OPENRICHARD_1_2 = 41232
    USER_IS_NOT_CAPABLE_FOR_GEOLOCATION_PUSH_REQUEST = 41240
    FAILED_TO_GET_MESSAGE_CONTENT_TYPE = 41250
    INVALID_STATE = 42001
    INVALID_MESSAGE = 42002
    INVALID_DATE_TIME_FORMAT = 42003
    MISSING_CONTACT = 42004
    INVALID_CONTACT = 42005
    EMULATOR_ACCESS_ONLY = 42006
    CONTACT_NOT_IN_WHITELIST = 42007
    MISSING_MESSAGE_CONTENT = 42008
    INVALID_MESSAGE_CONTENT = 42009
    AMBIGUOUS_MESSAGE = 42010
    INVALID_MESSAGE_STATUS = 42011
    INVALID_ISTYPING_STATUS = 42012
    INVALID_TRAFFIC_TYPE = 42013
    INVALID_SUGGESTED_CHIPLIST_ASSOCIATION = 42014
    EMPTY_TEXT_MESSAGE = 42015
    MISSING_RICHCARD = 42031
    AMBIGUOUS_RICHCARD = 42032
    TOO_MANY_RICHCARDS = 42033
    MISSING_RICHCARD_LAYOUT = 42034
    MISSING_RICHCARD_CONTENT = 42035
    INVALID_CARDORIENTATION = 42036
    MISSING_IMAGE_ALIGNMENT = 42037
    INVALID_IMAGE_ALIGNMENT = 42038
    REDUNDANT_IMAGE_ALIGNMENT = 42039
    INVALID_RICHCARD_CAROUSEL_CARDWIDTH = 42040
    MISMATCHED_MEDIA_HEIGHT = 42041
    INVALID_RICHCARD_CONTENT = 42042
    INVALID_SUGGESTIONS = 42043
    TOO_MANY_SUGGESTIONS = 42044
    INVALID_SUGGESTION = 42045
    AMBIGUOUS_SUGGESTION = 42046
    AMBIGUOUS_SUGGESTED_ACTION = 42047
    TOO_MANY_SUGGESTIONS_IN_RICHCARD_MAX_4 = 42048
    TOO_MANY_SUGGESTION_DATA_MAX_2048 = 42049
    INVALID_ACTION = 42050
    INVALID_LOCATION_A = 42051  # Either latitude and longitude or query are required in location.
    AMBIGUOUS_LOCATION = 42052
    INVALID_MAPACTION = 42053
    AMBIGUOUS_MAPACTION = 42054
    INVALID_DIALERACTION = 42055
    AMBIGUOUS_DIALERACTION = 42056
    INVALID_COMPOSEACTION = 42057
    AMBIGUOUS_COMPOSEACTION = 42058
    INVALID_SETTINGSACTION = 42059
    AMBIGUOUS_SETTINGSACTION = 42060
    INVALID_CLIPBOARDACTION = 42061
    MISSING_LOCALBROWSERACTION = 42062
    INVALID_SHAREACTION = 42063
    AMBIGUOUS_SHAREACTION = 42064
    INVALID_THUMBNAIL = 42100
    MISSING_FILEURL = 42101
    MISSING_AUDIO_FILEURL = 42102
    MISSING_POS = 42103
    MISSING_MEDIA_INFORMATION_A = 42104
    MISSING_MEDIA_INFORMATION_B = 42105
    MISSING_MEDIA_INFORMATION_C = 42106
    MISSING_MEDIA_INFORMATION_D = 42107
    INVALID_LOCATION_B = 42201  # geolocationPushMessage should not contain label length > 200 Characters.
    INVALID_MEDIA_CONTENT_DESCRIPTION = 42202
    INVALID_MEDIA_TITLE = 42203
    INVALID_MEDIA_DESCRIPTION = 42204
    TOO_MANY_CONTENTS_IN_RICHARD_CAROUSEL = 42205
    INVALID_MEDIA_FILE_SIZE = 42206
    INVALID_EXPIRY_TIME_FORMAT = 42207
    INVALID_EXPIRY_TIME = 42208
    AMBIGUOUS_OPENRICHCARD = 42301
    MISSING_OPENRICHCARD = 42302
    MISSING_OPENRICHCARD_LAYOUT_WIDGET = 42303
    MISSING_OPENRICHCARD_VIEW_CONTENT = 42304
    MISSING_OPENRICHCARD_LINEARLAYOUT_CONTENT = 42305
    MISSING_OPENRICHCARD_TEXTVIEW_CONTENT = 42306
    INVALID_CONTENTS_IN_OPENRICHCARD_TEXTVIEW = 42307
    INVALID_TEXT_LENGTH_IN_OPENRICHCARD_TEXTVIEW = 42308
    MISSING_OPENRICHCARD_IMAGEVIEW_CONTENT = 42309
    TOO_SMALL_MEDIA_IN_OPENRICHCARD_IMAGEVIEW = 42310
    INVALID_OPENRICHCARD_IMAGEVIEW_SCALETYPE = 42311
    MISSING_OPENRICHCARD_WIDTH_OR_HEIGHT = 42312
    INVALID_OPENRICHCARD_WIDTH_OR_HEIGHT = 42313
    INVALID_OPENRICHCARD_COMMON_CONTENTS = 42314
    TOO_MANY_CHILD_IN_OPEN_RICH_CARD = 42315
    INVALID_FILE_TYPE = 42401
    DOWNLOAD_FAILURE = 42402
    MISSING_CONTACT_A = 42501
    MISSING_CONTENT_B = 42502
    MISSING_TITLE = 42503
    MISSING_DESCRIPTION = 42504
    MISSING_IMAGE_URL = 42505
    MISSING_IMAGE_TYPE = 42506
    MISSING_BUTTON_LINK = 42507
    MISSING_BUTTON_TEXT = 42508
    INVALID_TITLE = 42509
    INVALID_DESCRIPTION = 42510
    INVALID_IMAGE_URL_ADDRESS = 42511
    INVALID_BUTTON_URL_ADDRESS = 42512
    INVALID_BUTTON_TEXT = 42513
    DUPLICATED_MESSAGE_ID = 42514
    TOO_MANY_REQUEST = 42601


# class RcsException(Exception):
#     """
#     Base class for all RCSErrorCode exceptions.
#     """

#     def __init__(self, message):
#         self.message = message
#         super().__init__(message)

#     def __str__(self):
#         return self.message



from typing import Optional, Union

from rcs_pydantic import enums

from . import scheme


class RcsMessage:
    STANDALONE_1 = "SCS00000"  # 기본 말풍선
    STANDALONE_2 = "SCL00000"  # 텍스트 카드
    CAROUSEL_MEDIUM_1 = "CCwMhM0200"  # 슬라이드형(Medium, 2장)
    CAROUSEL_MEDIUM_2 = "CCwMhM0300"  # 슬라이드형(Medium, 3장)
    CAROUSEL_MEDIUM_3 = "CCwMhM0400"  # 슬라이드형(Medium, 4장)
    CAROUSEL_MEDIUM_4 = "CCwMhM0500"  # 슬라이드형(Medium, 5장)
    CAROUSEL_MEDIUM_5 = "CCwMhM0600"  # 슬라이드형(Medium, 6장)
    CAROUSEL_SMALL_1 = "CCwShS0200"  # 슬라이드형(Small, 2장)
    CAROUSEL_SMALL_2 = "CCwShS0300"  # 슬라이드형(Small, 3장)
    CAROUSEL_SMALL_3 = "CCwShS0400"  # 슬라이드형(Small, 4장)
    CAROUSEL_SMALL_4 = "CCwShS0500"  # 슬라이드형(Small, 5장)
    CAROUSEL_SMALL_5 = "CCwShS0600"  # 슬라이드형(Small, 6장)
    STANDALONE_MEDIA_TOP = "SCwThT00"  # 세로형(Tall)
    STANDALONE_MEDIA_TOP = "SCwThM00"  # 세로형(Medium)

    def __init__(
        self,
        message_info: scheme.MessageInfo,
        agency_id: str = "ktbizrcs",
        message_base_id: str = STANDALONE_1,
        service_type: str = "RCSSMS",
        expiry_option: int = 2,
        header: str = "0",
        footer: str = "080-0000-0000",
        cdr_id: str = "ktrcs02",
        copy_allowed: bool = True,
        body: Union[
            scheme.RcsSMSBody,
            scheme.RcsLMSBody,
            scheme.RcsMMSBody,
            scheme.RcsCHATBody,
            scheme.RcsTMPLBody,
        ] = ...,
        buttons: Optional[list] = None,
    ):
        self.message_info = message_info
        self.agency_id = agency_id
        self.message_base_id = message_base_id
        self.service_type = service_type
        self.expiry_option = expiry_option
        self.header = header
        self.footer = footer
        self.cdr_id = cdr_id
        self.copy_allowed = copy_allowed
        self.body = body
        self.buttons = buttons
        self.send_info = scheme.SendInfo(
            common=self.make_common_info(message_info), rcs=self.make_rcs_info(message_info)
        )

    def make_common_info(self, message_info: scheme.MessageInfo) -> scheme.CommonInfo:
        return scheme.CommonInfo(
            msgId=message_info.replyId,
            userContact=message_info.userContact,
            scheduleType=0,
            msgServiceType=enums.MessageServiceTypeEnum.RCS,
        )

    def make_rcs_info(self, message_info: scheme.MessageInfo) -> scheme.RcsInfo:
        rcs_info = scheme.RcsInfo(
            chatbotId=message_info.chatbotId,
            agencyId=self.agency_id,
            messagebaseId=self.message_base_id,
            serviceType=self.service_type,
            expiryOption=self.expiry_option,
            header=self.header,
            footer=self.footer,
            cdrId=self.cdr_id,
            copyAllowed=self.copy_allowed,
            body=self.body,
        )

        if self.buttons:
            rcs_info.buttons = self.buttons
        return rcs_info

    def send(self):
        self.send_info



from enum import Enum, IntEnum


class MessageServiceTypeEnum(str, Enum):
    RCS: str = "rcs"
    LEGACY: str = "rcs, legacy"  # rcs 부달시, xMS로fallback전송


class ServiceTypeEnum(str, Enum):
    SMS: str = "RCSSMS"
    LMS: str = "RCSLMS"
    MMS: str = "RCSMMS"
    TMPL: str = "RCSTMPL"
    CHAT: str = "RCSCHAT"


class LegacyServiceTypeEnum(str, Enum):
    SMS: str = "SMS"
    LMS: str = "LMS"


class ScheduleTypeEnum(IntEnum):
    IMMEDIATE: int = 0


class ExpiryOptionEnum(IntEnum):
    AFTER_THREE_DAYS: int = 1
    AFTER_SETTING_TIMES: int = 2


class HeaderEnum(str, Enum):
    """
    헤더 광고에 대한 정의
    0: 광고 넣지않음
    1: 광고 넣음
    """

    NOT_ADVERTISE: str = "0"
    ADVERTISE: str = "1"


class ActionEnum(str, Enum):
    URL_ACTION: str = "urlAction"  # 단말기에 기본 웹 브라우저로 설정된 앱을 통해서, 웹페이지로 이동할 수있습니다
    LOCAL_BROWSER_ACTION: str = "localBrowserAction"  # 단말기의 메시지 앱 내부 브라우저를 통해 웹페이지로 이동할 수 있습니다
    MAP_ACTION: str = "mapAction"  # 미리 지정된 위치를 보여주거나 사용자의 현재 위치를 서버로 전송 할 수 있습니다.
    CALENDAR_ACTION: str = "calendarAction"  # 사용자의 캘린더에 특정 일정을 등록 할 수 있습니다.
    CLIPBOARD_ACTION: str = "clipboardAction"  # 특정 문구를 사용자 단말이 자동으로 복사 할 수 있게 합니다
    COMPOSE_ACTION: str = "composeAction"  # 다른 번호로 메시지를 보낼 수 있도록 대화방을 엽니다.
    DIALER_ACTION: str = "dialerAction"  # 특정 전화번호로 전화를 걸 수 있습니다.


class MessageStatusEnum(str, Enum):
    SUCCESS: str = "success"
    FAIL: str = "fail"


class MnoInfoEnum(str, Enum):
    KT: str = "KT"
    SKT: str = "SKT"
    LGU: str = "LGU"


class BillEnum(IntEnum):
    FREE: int = 0
    CHARGE: int = 1


class EventTypeEnum(str, Enum):
    MESSAGE: str = "message"
    RESPONSE: str = "response"
    NEW_USER: str = "newuser"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rcs_pydantic']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'rcs-pydantic',
    'version': '0.1.9',
    'description': '',
    'long_description': '<p align="center">\n    <em>한국 통신사 rcs 를 위한 pydantic 구조체</em>\n</p>\n<p align="center">\n<a href="https://github.com/xncbf/rcs-pydantic/actions?query=workflow%3ATests+event%3Apush+branch%3Amain" target="_blank">\n    <img src="https://github.com/xncbf/rcs-pydantic/workflows/Tests/badge.svg?event=push&branch=main" alt="Test">\n</a>\n<a href="https://codecov.io/gh/xncbf/rcs-pydantic" target="_blank">\n    <img src="https://img.shields.io/codecov/c/github/xncbf/rcs-pydantic?color=%2334D058" alt="Coverage">\n</a>\n<a href="https://pypi.org/project/rcs-pydantic" target="_blank">\n    <img src="https://img.shields.io/pypi/v/rcs-pydantic?color=%2334D058&label=pypi%20package" alt="Package version">\n</a>\n<a href="https://pypi.org/project/rcs-pydantic" target="_blank">\n    <img src="https://img.shields.io/pypi/pyversions/rcs-pydantic.svg?color=%2334D058" alt="Supported Python versions">\n</a>\n</p>\n\n- [Installation](#installation)\n- [Quick start](#quick-start)\n- [Features](#features)\n- [Contribution](#contribution)\n\n## Installation\n\n```sh\npip install rcs-pydantic\n```\n\n## Quick start\n\n```py\nimport httpx\nfrom fastapi import Body, FastAPI\nfrom rcs_pydantic import scheme\n\napp = FastAPI()\n\n\ndef get_card(message_info):\n    rcs_message = RcsMessage(\n        message_info,\n        agency_id="ktbizrcs",\n        message_base_id="STANDALONE_1",\n        service_type="RCSSMS",\n        expiry_option=2,\n        header="0",\n        footer="080-0000-0000",\n        cdr_id="ktrcs02",\n        copy_allowed=True,\n        body=scheme.RcsSMSBody(\n            description=textwrap.dedent(\n                """\\\n                안녕하세요.\n                메세지입니다.\n                """\n            )\n        ),\n        buttons=[\n            scheme.ButtonInfo(\n                suggestions=[\n                    scheme.SuggestionInfo(\n                        action=scheme.ActionInfo(\n                            urlAction=scheme.UrlActionInfo(\n                                openUrl=scheme.OpenUrlInfo(url="https://www.kt.com")\n                            ),\n                            displayText="kt 홈페이지 들어가기",\n                            postback=scheme.PostbackInfo(data="postback_kt"),\n                        )\n                    )\n                ]\n            )\n        ],\n    )\n    return rcs_message\n\n@app.post("/corp/{version}/momsg")\nasync def recieve_message(version: str, message_info: scheme.MessageInfo = Body(...)):\n    """\n    메세지 받는 웹훅\n    """\n    body = get_card(message_info)\n    response = httpx.post(url=f"{config.RCS_URL}/message", json=body.json())\n    return {"status": response.status_code, "content": response.json()}\n\n```\n\n## Features\n\nTODO\n\n## Contribution\n\nTODO\n',
    'author': 'xncbf',
    'author_email': 'xncbf12@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/xncbf/rcs-pydantic',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
